<?php

namespace Drupal\raven\Commands;

use Consolidation\AnnotatedCommand\CommandData;
use Drupal\raven\EventSubscriber\RequestSubscriber;
use Drupal\raven\Logger\Raven;
use Drush\Commands\DrushCommands;
use Sentry\SentrySdk;
use Sentry\Severity;
use Sentry\Tracing\SpanContext;

/**
 * Provides Drush commands for Raven module.
 */
class RavenCommands extends DrushCommands {

  /**
   * The @raven.request_subscriber service.
   *
   * @var \Drupal\raven\EventSubscriber\RequestSubscriber|null
   */
  protected $eventSubscriber;

  /**
   * The @logger.raven service.
   *
   * @var \Drupal\raven\Logger\Raven|null
   */
  protected $ravenLogger;

  /**
   * Injects Raven event subscriber.
   */
  public function setEventSubscriber(RequestSubscriber $event_subscriber): void {
    $this->eventSubscriber = $event_subscriber;
  }

  /**
   * Injects Raven logger service.
   */
  public function setRavenLogger(Raven $raven_logger): void {
    $this->ravenLogger = $raven_logger;
  }

  /**
   * Sets up Drush command performance tracing.
   *
   * @hook pre-command *
   */
  public function preCommand(CommandData $commandData): void {
    if ($this->eventSubscriber) {
      $this->eventSubscriber->onConsoleCommand($commandData);
    }
  }

  /**
   * Send a test message to Sentry.
   *
   * Because messages are sent to Sentry asynchronously, there is no guarantee
   * that the message was actually delivered successfully.
   *
   * @param string $message
   *   The message text.
   * @param mixed[] $options
   *   An associative array of options.
   *
   * @option level
   *   The message level (debug, info, warning, error, fatal).
   *
   * @command raven:captureMessage
   * @usage drush raven:captureMessage
   *   Send test message to Sentry.
   * @usage drush raven:captureMessage --level=error
   *   Send error message to Sentry.
   * @usage drush raven:captureMessage 'Mic check.'
   *   Send "Mic check" message to Sentry.
   */
  public function captureMessage(string $message = 'Test message from Drush.', array $options = [
    'level' => 'info',
  ]): void {
    if (!$this->ravenLogger) {
      throw new \Exception('Sentry logger service not available.');
    }
    if (!$this->ravenLogger->getClient()) {
      throw new \Exception('Sentry client not available.');
    }

    $severity = new Severity($options['level']);

    $start = microtime(TRUE);

    $id = \Sentry\captureMessage($message, $severity);

    if ($parent = SentrySdk::getCurrentHub()->getSpan()) {
      $span = new SpanContext();
      $span->setOp('sentry.capture');
      $span->setDescription("$severity: $message");
      $span->setStartTimestamp($start);
      $span->setEndTimestamp(microtime(TRUE));
      $parent->startChild($span);
    }

    if (!$id) {
      throw new \Exception('Send failed.');
    }
    $this->logger()->success(dt('Message sent as event %id.', ['%id' => $id]));
  }

}
