<?php

namespace Drupal\raven\Http;

use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Promise\Create;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Sentry\Breadcrumb;
use Sentry\SentrySdk;
use Sentry\Tracing\SpanContext;
use Sentry\Tracing\SpanStatus;

use function GuzzleHttp\Promise\rejection_for;

/**
 * Instrument Guzzle HTTP requests.
 */
class HttpClientMiddleware {

  /**
   * {@inheritdoc}
   */
  public function __invoke(): callable {
    return function ($handler) {
      return function (RequestInterface $request, array $options) use ($handler) {
        $hub = NULL;
        $span = NULL;
        if (class_exists(SentrySdk::class)) {
          $hub = SentrySdk::getCurrentHub();
          if ($parent = $hub->getSpan()) {
            $context = new SpanContext();
            $context->setOp('http.client');
            $context->setDescription($request->getMethod() . ' ' . $request->getUri());
            $span = $parent->startChild($context);
            $request = $request->withHeader('sentry-trace', $span->toTraceparent());
          }
        }
        $handlerPromiseCallback = static function ($responseOrException) use ($hub, $request, $span) {
          if ($hub) {
            if ($span) {
              $span->finish();
            }
            $response = NULL;
            if ($responseOrException instanceof ResponseInterface) {
              $response = $responseOrException;
            }
            elseif ($responseOrException instanceof RequestException) {
              $response = $responseOrException->getResponse();
            }
            $breadcrumbData = [
              'url' => (string) $request->getUri(),
              'method' => $request->getMethod(),
              'request_body_size' => $request->getBody()->getSize(),
            ];
            if ($response) {
              if ($span) {
                $span->setStatus(SpanStatus::createFromHttpStatusCode($response->getStatusCode()));
              }
              $breadcrumbData['status_code'] = $response->getStatusCode();
              $breadcrumbData['response_body_size'] = $response->getBody()->getSize();
            }
            elseif ($span) {
              $span->setStatus(SpanStatus::internalError());
            }
            $hub->addBreadcrumb(new Breadcrumb(Breadcrumb::LEVEL_INFO, Breadcrumb::TYPE_HTTP, 'http', NULL, $breadcrumbData));
          }
          if ($responseOrException instanceof \Throwable) {
            // @phpstan-ignore-next-line for compatibility with older Guzzle versions.
            return class_exists(Create::class) ? Create::rejectionFor($responseOrException) : rejection_for($responseOrException);
          }
          return $responseOrException;
        };
        return $handler($request, $options)->then($handlerPromiseCallback, $handlerPromiseCallback);
      };
    };
  }

}
